//
//  SimonGameViewController.m
//  SimonGame
//
//  Created by Spencer Hakim on 8/2/10.
//  Copyright Spencer Hakim 2010. All rights reserved.
//  License: http://www.gnu.org/licenses/gpl-3.0.txt
//

#import "SimonGameViewController.h"

//ROUNDLENGTH represents how many additions are made to the pattern before waiting for user input
#define ROUNDLENGTH 2

//INTERVAL is how long each pattern segment is displayed for
#define INTERVAL 0.75

@implementation SimonGameViewController

@synthesize red, blue, green, yellow;

- (IBAction)redPressed {
	//make sure that we are currently accepting user input
	if( [pattern length] % ROUNDLENGTH != 0 || [pattern length] == 0 || repeatIndex != 0 )
		return;
	
	[self checkButtonPress:@"R"];
}

- (IBAction)bluePressed {
	//make sure that we are currently accepting user input
	if( [pattern length] % ROUNDLENGTH != 0 || [pattern length] == 0 || repeatIndex != 0 )
		return;
	
	[self checkButtonPress:@"B"];
}

- (IBAction)greenPressed {
	//make sure that we are currently accepting user input
	if( [pattern length] % ROUNDLENGTH != 0 || [pattern length] == 0 || repeatIndex != 0 )
		return;
	
	[self checkButtonPress:@"G"];
}

- (IBAction)yellowPressed {
	//make sure that we are currently accepting user input
	if( [pattern length] % ROUNDLENGTH != 0 || [pattern length] == 0 || repeatIndex != 0 )
		return;
	
	[self checkButtonPress:@"Y"];
}

//callback from the Game Over alert view
- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {
	//reset our variables
	repeatIndex = userIndex = 0;
	[pattern setString:@""];
	
	//start a new game in 1 second
    [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(changeColor:) userInfo:nil repeats:NO];
}

//validate the button press of a user to make sure it conforms to the pattern
- (void)checkButtonPress:(NSString *)val {
	if( [pattern characterAtIndex:userIndex] != [val characterAtIndex:0] ) { //if the user entered the wrong color for this position in the pattern, show the Game Over alert view
		UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:@"Game Over" message:@"Click OK to try again." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
		[alert show];
	}
	else { //user entered correct color, let's move on to the next color in the pattern, or move on to the next round
		userIndex++;
		
		if( userIndex >= [pattern length] )
			[NSTimer scheduledTimerWithTimeInterval:INTERVAL target:self selector:@selector(repeatPattern:) userInfo:nil repeats:NO];
	}
}

//generates the next color in the pattern and displays it
- (void)changeColor:(NSTimer *)timer {
	[self fadeOut:timer];
	
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.25];
	switch( arc4random() % 4 )
	{
		case 0:
			[red setAlpha:1.0f];
			[pattern appendString:@"R"];
			break;
			
		case 1:
			[green setAlpha:1.0f];
			[pattern appendString:@"G"];
			break;
			
		case 2:
			[blue setAlpha:1.0f];
			[pattern appendString:@"B"];
			break;
			
		case 3:
			[yellow setAlpha:1.0f];
			[pattern appendString:@"Y"];
			break;
			
		default:
			break;
	}
	[UIView commitAnimations];
	
	//print current pattern in Console, make sure repeatIndex and userIndex are 0
	NSLog(@"%@", pattern);
	repeatIndex = userIndex = 0;
	
	if( [pattern length] % ROUNDLENGTH != 0 ) //if we haven't yet reached ROUNDLENGTH, we need to generate more colors for the pattern
		[NSTimer scheduledTimerWithTimeInterval:INTERVAL target:self selector:@selector(changeColor:) userInfo:nil repeats:NO];
	else //we need to fade out all the buttons, since we have generated enough new colors
		[NSTimer scheduledTimerWithTimeInterval:INTERVAL target:self selector:@selector(fadeOut:) userInfo:nil repeats:NO];
}

//displays the portion of the pattern that has already been generated
- (void)repeatPattern:(NSTimer *)timer {
	[self fadeOut:timer];
	
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.25];
	switch( [pattern characterAtIndex:repeatIndex] )
	{
		case 'R':
			[red setAlpha:1.0f];
			break;
			
		case 'G':
			[green setAlpha:1.0f];
			break;
			
		case 'B':
			[blue setAlpha:1.0f];
			break;
			
		case 'Y':
			[yellow setAlpha:1.0f];
			break;
			
		default:
			break;
	}
	[UIView commitAnimations];
	
	//moves on to the next color in the index and makes sure userIndex is set to 0
	repeatIndex++;
	userIndex = 0;
	
	if( repeatIndex < [pattern length] ) //we still have more pattern to repeat
		[NSTimer scheduledTimerWithTimeInterval:INTERVAL target:self selector:@selector(repeatPattern:) userInfo:nil repeats:NO];
	else //we need to start generating new colors
		[NSTimer scheduledTimerWithTimeInterval:INTERVAL target:self selector:@selector(changeColor:) userInfo:nil repeats:NO];
}

//Fades all buttons to their default alpha of 0.5f over 250ms
- (void)fadeOut:(NSTimer *)timer {
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.25];
	[red setAlpha:0.5f];
	[green setAlpha:0.5f];
	[blue setAlpha:0.5f];
	[yellow setAlpha:0.5f];
	[UIView commitAnimations];
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	
	[NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(changeColor:) userInfo:nil repeats:NO]; //wait one second before starting
	
	//initialize our variables
	pattern = [[NSMutableString alloc] initWithString:@""];
	repeatIndex = userIndex = 0;
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
	[pattern release];
}

- (void)dealloc {
    [super dealloc];
}

@end
